// some RDF helper functions

const GENERAL_NS="http://www.xulplanet.com/rdf/reobase#";
const RP_NS="http://www.xulplanet.com/ns/rpath-template";
const RDF_NS="http://www.w3.org/1999/02/22-rdf-syntax-ns#";
const RDFS_NS="http://www.w3.org/2000/01/rdf-schema#";
const XUL_NS="http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var RDF="@mozilla.org/rdf/rdf-service;1";
RDF=Components.classes[RDF].getService();
RDF=RDF.QueryInterface(Components.interfaces.nsIRDFService);

var lastDatasource = null;

function RDFAssert(ds,subj,prop,obj)
{
  ds.Assert(subj,RDF.GetResource(prop),obj,true);
}
 
function RDFUnassert(ds,subj,prop,obj)
{
  prop=RDF.GetResource(prop);
  if (obj == null) obj=ds.GetTarget(subj,prop,true);
  if (obj != null) ds.Unassert(subj,prop,obj,true);
}

function RDFAssertAll(ds,subj,arr)
{
  for (var t=0; t<arr.length; t+=2){
    ds.Assert(subj,RDF.GetResource(arr[t]),RDF.GetLiteral(arr[t+1]),true);
  }
}

function RDFUnassertAll(ds,subj,removeChildren)
{
  if (typeof subj == "string") subj=RDF.GetResource(subj);
  var iter=ds.ArcLabelsOut(subj);
  while (iter.hasMoreElements()){
    var prop=iter.getNext();
    var obj=ds.GetTarget(subj,prop,true);
    ds.Unassert(subj,prop,obj,true);
  }

  if (removeChildren){
    var container=RDFGetContainer(ds,subj);
    if (container != null){
      var children=container.GetElements();
      while (children.hasMoreElements()){
        var child=children.getNext();
        RDFUnassertAll(ds,child,false);
      }
    }
  }
}

function RDFGetValue(node)
{
  try {
    return node.QueryInterface(Components.interfaces.nsIRDFResource).Value;
  }
  catch (ex){
    try {
      return node.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
    }
    catch (ex){
    }
  }
  return "";
}

function RDFGetResource(ds,subj,prop)
{
  var obj=ds.GetTarget(subj,RDF.GetResource(prop),true);
  try {
    return obj.QueryInterface(Components.interfaces.nsIRDFResource);
  }
  catch (ex){
    return null;
  }
}

function RDFGetLiteral(ds,subj,prop)
{
  var obj=ds.GetTarget(subj,RDF.GetResource(prop),true);
  try {
    return obj.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
  }
  catch (ex){
    return null;
  }
}

function RDFGetContainer(ds,node)
{
  var RDFContainer = '@mozilla.org/rdf/container;1';
  RDFContainer = Components.classes[RDFContainer].createInstance();
  RDFContainer = RDFContainer.QueryInterface(Components.interfaces.nsIRDFContainer);
 
  try {
    RDFContainer.Init(ds,node);
    return RDFContainer;
  }
  catch (ex){
    return null;
  }
}

function addDataSource(ds)
{
  if (!gGeneralDatasource){
    gGeneralDatasource=Components.classes["@mozilla.org/rdf/datasource;1?name=composite-datasource"]
                         .createInstance(Components.interfaces.nsIRDFCompositeDataSource);
  }
  gGeneralDatasource.AddDataSource(ds);
  lastDatasource = ds;
}

function isDataSourceLoaded(dsuri)
{
  var iter = gGeneralDatasource.GetDataSources();
  while (iter.hasMoreElements()){
    var ds = iter.getNext().QueryInterface(Components.interfaces.nsIRDFDataSource);
    if (ds.URI == dsuri) return true;
  }
  return false;
}

function createRPathTemplateBuilder(ds,defns,listener)
{
  var builder=Components.classes["@xulplanet.com/rdf/rpath-templatebuilder;1"]
              .createInstance(Components.interfaces.rpIRPathTemplateBuilder);

  if (ds) builder.dataSource=ds;
  if (defns) builder.defaultNamespace=defns;
  if (listener) builder.listener=listener;
  return builder;
}

function evaluateRPath(ds,defns,context,expr)
{
  var evaluator=Components.classes["@xulplanet.com/rdf/rpath-evaluator;1"]
                .createInstance(Components.interfaces.rpIRPathEvaluator);

  if (ds) evaluator.dataSource=ds;
  if (defns) evaluator.defaultNamespace=defns;
  if (context) evaluator.setSingleContextNode(context);
  if (expr) evaluator.expression=expr;
  return evaluator.evaluate(1);
}

function rebuildPage(builder,elem,context)
{
  builder.parseTemplate(elem,context);
}

function serializeRDF(ds)
{
  var outputStream = {
    data: "",
    close : function(){},
    flush : function(){},
    write : function (buffer,count){
      this.data += buffer;
      return count;
    },
    writeFrom : function (stream,count){},
    isNonBlocking: false
  }
  serializeRDFToStream(ds,outputStream);
  return outputStream.data;
}

function serializeRDFToStream(ds,outputStream)
{
  var ser=Components.classes["@mozilla.org/rdf/xml-serializer;1"]
                    .createInstance(Components.interfaces.nsIRDFXMLSerializer);
  ser.init(ds);
  ser.QueryInterface(Components.interfaces.nsIRDFXMLSource).Serialize(outputStream);
}
