const ONTOLOGY_DSURI = "chrome://topicalla/content/ontology.rdf";
const FOAFSCHEMA_DSURI = "chrome://topicalla/content/foaf-schema.rdf";

// XXX {a}:b syntax causes crash

var gInited=false;
var gGeneralDatasource=null;
var gGeneralBuilder=null;
var gStoreInHistory=true;

var contentArea=null;

function TopicallaState()
{
  this.pageBuilder = null;
  this.bookmark = null;
  this.displayType = null;
  this.layout = null;
  this.dsuri = null;
  this.resource = null;
  this.history = [];
  this.historyIndex = -1;
}

function getState()
{
  return contentArea.selectedBrowser.topicallaState;
}

var subblockListener = {
  constructSubblock: function(node,displayMode,context){
    var layout;
    var state = getState();
    var property = node.getAttributeNS(RP_NS,"property");
    if (property){
      context = RDFGetLiteral(gGeneralDatasource,RDF.GetResource(property),
                              RDFS_NS + "range");
      if (context){ context = RDF.GetResource(context); }
      layout = determineLayout(context,displayMode,true);
    }
    else {
      layout = determineLayout(context,displayMode,false);
    }
    var binding = "-moz-binding: url(" + layout + ");";
    if (binding == node.getAttribute("style")){
      state.pageBuilder.parseAnonymousTemplate(node,context);
      return 3;
    }
    if (layout){
      node.setAttribute("style","-moz-binding: url(" + layout + ");");
    }
    return 7;
  }
};

var layoutProgressListener =
{
  onProgressChange : function (aWebProgress, aRequest,
                               aCurSelfProgress, aMaxSelfProgress,
                               aCurTotalProgress, aMaxTotalProgress){},
  onStateChange : function(aWebProgress, aRequest, progressstate, status)
  {
    if (!((progressstate & Components.interfaces.nsIWebProgressListener.STATE_STOP) &&
          (progressstate & Components.interfaces.nsIWebProgressListener.STATE_IS_WINDOW))) return;

    var cdoc = contentArea.contentDocument;
    var state = getState();
    cdoc.RPathProcessor = {
      recalculateNode : function (baseElem,resource){
        rebuildPage(state.pageBuilder,baseElem,resource);
      },
      recalculateAnonymousNode : function (baseElem,resource){
        if (!resource){
          resource = state.pageBuilder.getResultForNode(baseElem);
        }
        state.pageBuilder.parseAnonymousTemplate(baseElem,resource);
      },
      getContextForNode : function (elem){
        return state.pageBuilder.getContextForNode(elem);
      },
      getHTMLDocument : function (){ return window.frames["blankhtml"].document; },
      addSubHistoryItem: addSubHistoryItem,
      goLocalNode: goLocalNode,
      goNode: goNode
    };

    recalculatePage(getState().resource);
  },
  onLocationChange : function(aWebProgress, aRequest, aLocation){},
  onStatusChange : function(aWebProgress, aRequest, aStatus, aMessage){},
  onSecurityChange : function(aWebProgress, aRequest, aState){}
};

function Init()
{
  if (!gInited){
    gInited=true;

    contentArea=document.getElementById("appcontent");

    addDataSource(RDF.GetDataSourceBlocking(ONTOLOGY_DSURI));
    gGeneralBuilder=createRPathTemplateBuilder(gGeneralDatasource,GENERAL_NS,null);

//    addDataSource(RDF.GetDataSourceBlocking(FOAFSCHEMA_DSURI));

    var state = new TopicallaState();
    contentArea.selectedBrowser.topicallaState = state;
    state.pageBuilder=createRPathTemplateBuilder(gGeneralDatasource,null,subblockListener);

    contentArea.addProgressListener(layoutProgressListener,
      Components.interfaces.nsIWebProgress.NOTIFY_STATE);

    goBookmark(RDF.GetResource("urn:topicalla:userinfo:startpage"));

    var sidebar=document.getElementById("bookmarksSidebar");
    rebuildPage(gGeneralBuilder,sidebar,null);
  }
}

function updateFocusMenuItems()
{
  goUpdateCommand('cmd_undo');
  goUpdateCommand('cmd_redo');
  goUpdateCommand('cmd_cut');
  goUpdateCommand('cmd_copy');
  goUpdateCommand('cmd_paste');
  goUpdateCommand('cmd_delete');
}

function updateSelectMenuItems()
{
  goUpdateCommand('cmd_cut');
  goUpdateCommand('cmd_copy');
  goUpdateCommand('cmd_delete');
}

function updateUndoMenuItems()
{
  goUpdateCommand('cmd_undo');
  goUpdateCommand('cmd_redo');
}

function updatePasteMenuItems()
{
  goUpdateCommand('cmd_paste');
}

function doNewTab()
{
  contentArea.selectedTab = contentArea.addTab("about:blank");
  var state = new TopicallaState();
  contentArea.selectedBrowser.topicallaState = state;
  state.pageBuilder=createRPathTemplateBuilder(null,null,subblockListener);
}

function addBookmarkCallback(bookmarkObj)
{
  if (bookmarkObj.resid && bookmarkObj.dsuri){
    addBookmark(RDF.GetResource(bookmarkObj.resid),bookmarkObj.dsuri,
                RDF.GetResource("urn:topicalla:userinfo:bookmarks"),"bookmarks");
  }
}

function doAddBookmark()
{
  var state = getState();
  var type = RDFGetResource(gGeneralDatasource,state.resource,RDF_NS + "type");

  var defaultsObj = {
    name: document.getElementById("currentNode").value,
    resid: state.resource.Value,
    dsuri: state.dsuri,
    type: type,
    generalDS: gGeneralDatasource
  };

  window.openDialog("addbookmark.xul","_blank","chrome",addBookmarkCallback,defaultsObj);
}

function doEditLayouts()
{
  window.open("editlayouts.xul","_blank","chrome");
}

function doViewSource()
{
  window.open("view-source:"+getState().layout,"_blank","chrome");
}

function doViewSourceData()
{
  window.open("viewsource.xul","_blank","chrome");
}

function doCustomizeView()
{
  if (hideShowSidebar('customizeViewSidebar')){
    var state = getState();
    var sidebar=document.getElementById("customizeViewSidebar");
    rebuildPage(gGeneralBuilder,sidebar,state.resource);
  }
}

function goItemInSidebar(evt)
{
  if (evt.target){
    item = gGeneralBuilder.getContextForNode(evt.target);
    if (item) goBookmark(item);
  }
}

function goBookmark(bookmarkID,resource)
{
  var state = getState();
  state.bookmark=bookmarkID;

  if (!resource){
    resource = RDFGetResource(gGeneralDatasource,bookmarkID,GENERAL_NS + "startNode");
  }
  var dsuri = RDFGetLiteral(gGeneralDatasource,bookmarkID,GENERAL_NS + "datasource");
  var layout = determineLayout(resource,"top",false);

  if (!dsuri){
    state.dsuri=null;
    loadLayout(resource,layout);
  }
  else {
    loadDataSource(resource,dsuri,layout);
  }
}

function goView(evt)
{
  gStoreInHistory=false;
  var layout = RDFGetLiteral(gGeneralDatasource,evt.target.value,GENERAL_NS + "display");
  loadLayout(getState().resource,layout);
}

function goLocalNode(resource,dsuri)
{
  if (!resource) return;
  if (typeof resource == "string") resource=RDF.GetResource(resource);

  if (!dsuri){
    recalculatePage(resource);
  }
  else {
    loadDataSource(resource,dsuri,getState().layout);
  }
}

function goNode(resource,dsuri)
{
  getState().bookmark=null;

//  if (!resource) return;
  if (typeof resource == "string") resource=RDF.GetResource(resource);

  if (!dsuri){
    loadLayout(resource,null);
  }
  else {
    loadDataSource(resource,dsuri,null);
  }
}

function loadDataSource(resource,dsuri,layout)
{
  if (dsuri.indexOf(":") == -1){
    var url=document.location.href;
    dsuri=url.substring(0,url.lastIndexOf("/")+1)+dsuri;
  }

  var state = getState()
  if (state.dsuri == dsuri){
    loadLayout(resource,layout);
    return;
  }
  state.dsuri=dsuri;

  if (isDataSourceLoaded(dsuri)){
    loadLayout(resource,layout);
  }
  else if (dsuri.indexOf("http") == 0){
    var xrequest=new XMLHttpRequest();
    xrequest.open("GET",dsuri,true);
    xrequest.onload=function (){
      var ios=Components.classes["@mozilla.org/network/io-service;1"]
                          .getService(Components.interfaces.nsIIOService);
      dsuri=ios.newURI(dsuri,null,null);
      var dsource=Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                            .createInstance(Components.interfaces.nsIRDFDataSource);
      var xmlParser=Components.classes["@mozilla.org/rdf/xml-parser;1"]
                              .createInstance(Components.interfaces.nsIRDFXMLParser);
      xmlParser.parseString(dsource,dsuri,xrequest.responseText);
      addDataSource(dsource);

      loadLayout(resource,layout);
    }
    xrequest.send(null);
    xrequest.overrideMimeType("text/xml");
  }
  else {
    if (dsuri != "rdf:this") addDataSource(RDF.GetDataSourceBlocking(dsuri));
    loadLayout(resource,layout);
  }
}

function loadLayout(resource,layout)
{
  var state = getState();
  state.resource=resource;

  if (!layout){
    // XXX Guess the layout to use based on the resource and datasource
    layout = determineLayout(resource,"top",false);
    resource=state.resource;
  }

  if (state.layout != layout){
    state.layout=layout;
    contentArea.loadURI(layout);
  }
  else {
    recalculatePage(resource);
  }
}

function determineLayout(resource,displayMode,istypenode)
{
  var displayType;

  var state = getState();
  if (!resource){
    // guess the root resource from the datasource
    var types = evaluateRPath(gGeneralDatasource,null,null,
      "rdfs:{Class}^*[contains(reo:displays%*/reo:variant,'" + displayMode + "')]");

    var typeProp = RDF.GetResource(RDF_NS+"type");
    var len = types.count;
    for (var t=0; t<len; t++){
      displayType = types.getResourceValue(t);
      resource = lastDatasource.GetSource(typeProp,displayType,true);
      if (resource){
        state.resource = resource;
        break;
      }
    }
    if (!resource) return;
  }
  else {
    try {
      resource=resource.QueryInterface(Components.interfaces.nsIRDFResource);
    } catch(e){
      return "chrome://topicalla/content/binds.xml#label";
    }

    displayType = (istypenode) ? resource :
                                 RDFGetResource(gGeneralDatasource,resource,RDF_NS + "type");
    if (!displayType) displayType = RDFGetResource(gGeneralDatasource,resource,RDF_NS + "type");
  }

  if (displayMode == "top") state.displayType = displayType;

  var firstItem = null;
  var useView = function(elem,variant,arg)
  {
    if (!firstItem) firstItem = elem;
    if (variant == displayMode) return RDFGetLiteral(gGeneralDatasource,elem,GENERAL_NS + "display");
    else return null;
  }

  var retval = null;
  while (displayType){
    retval = iterateOverViews(displayType,useView);
    if (retval) break;
    displayType = RDFGetResource(gGeneralDatasource,displayType,RDFS_NS + "subClassOf");
  }

  if (!retval && firstItem) retval = RDFGetLiteral(gGeneralDatasource,firstItem,GENERAL_NS + "display");
  if (!retval) retval = "chrome://topicalla/content/binds.xml#label";
  return retval;
}

function recalculatePage(resource)
{
  var state = getState();
  if (typeof resource == "string") resource=RDF.GetResource(resource);
  state.resource=resource;

  if (state.displayType){
    state.pageBuilder.defaultNamespace=
      RDFGetLiteral(gGeneralDatasource,state.displayType,GENERAL_NS+"defns");
  }
  else {
    state.pageBuilder.defaultNamespace="";
  }

//  dump("NS:"+state.pageBuilder.defaultNamespace+"\n");
//  dump("DSURI:"+state.dsuri+"]\n");
//  dump("RES:"+resource.Value+"\n");

  if (resource){
    rebuildPage(state.pageBuilder,contentArea.contentDocument.documentElement,resource);
    if (gStoreInHistory){
      state.historyIndex++;
      state.history.push(resource);
      addBookmark(resource,state.dsuri,RDF.GetResource("urn:topicalla:userinfo:history"),"history");
    }
  }
  gStoreInHistory=true;

  var backButton=document.getElementById("backButton");
  if (state.historyIndex > 0) backButton.removeAttribute("disabled");
  else backButton.setAttribute("disabled","true");

  var forwardButton=document.getElementById("forwardButton");
  if (state.historyIndex < state.history.length-1) forwardButton.removeAttribute("disabled");
  else forwardButton.setAttribute("disabled","true");

  var currentLabel = getNodeLabel(resource,state.displayType);
  if (!currentLabel) currentLabel="";
  document.getElementById("currentNode").value = currentLabel;

  document.getElementById("progress").setAttribute("mode","normal");
}

function getNodeLabel(resource,type)
{
  var labelProp = RDFGetResource(gGeneralDatasource,type,GENERAL_NS + "labelProp");
  if (labelProp){
    return RDFGetLiteral(gGeneralDatasource,resource,labelProp.Value);
  }
  return null;
}

function goHistory(dir)
{
  gStoreInHistory=false;
  var state = getState();
  state.historyIndex+=dir;
  var historyElem = state.history[state.historyIndex];
  loadLayout(historyElem);
}

function hideShowSidebar(panelid)
{
  var sidebar=document.getElementById("sidebar");
  var splitter=document.getElementById("sidebarSplitter");
  var panel=document.getElementById(panelid);

  if (sidebar.getAttribute("collapsed") == "true"){
    if (panel) sidebar.selectedPanel = sidebar;
    sidebar.removeAttribute("collapsed");
    splitter.removeAttribute("collapsed");
    return true;
  }
  else if (panel == sidebar.selectedPanel){
    sidebar.setAttribute("collapsed","true");
    splitter.setAttribute("collapsed","true");
    return false;
  }
  else {
    sidebar.selectedPanel = panel;
    return true;
  }
}

function addBookmark(resource,dsuri,container,type)
{
  var bookmarkContainer=RDFGetContainer(gGeneralDatasource,container);

  var startProp = RDF.GetResource(GENERAL_NS+"startNode");

  var bookmarkItem = null;
  var iter = bookmarkContainer.GetElements();
  while (iter.hasMoreElements()){
    var item = iter.getNext();
    if (gGeneralDatasource.HasAssertion(item,startProp,resource,true)){
      bookmarkItem = item;
      break;
    }
  }
  if (bookmarkItem == null){
    bookmarkItem = RDF.GetAnonymousResource();
    bookmarkContainer.AppendElement(bookmarkItem);
    RDFAssert(gGeneralDatasource,bookmarkItem,GENERAL_NS+"datasource",RDF.GetLiteral(dsuri));
    RDFAssert(gGeneralDatasource,bookmarkItem,GENERAL_NS+"startNode",resource);

    var type = RDFGetResource(gGeneralDatasource,resource,RDF_NS+"type");
    if (type){
      RDFAssert(gGeneralDatasource,resource,RDF_NS+"type",type);

      var label = getNodeLabel(resource,type);
      if (label){
        RDFAssert(gGeneralDatasource,bookmarkItem,GENERAL_NS+"name",RDF.GetLiteral(label));
      }
    }
  }

  var sidebar=document.getElementById(type+"Sidebar");
  rebuildPage(gGeneralBuilder,sidebar,null);
}

function removeFromHistory()
{
  var container=RDF.GetResource("urn:topicalla:userinfo:history");
  var bookmarkContainer=RDFGetContainer(gGeneralDatasource,container);

  var listbox=document.getElementById("historyList");
  for (var t=0; t<listbox.selectedCount; t++){
    var resource = gGeneralBuilder.getContextForNode(listbox.getSelectedItem(t));
    bookmarkContainer.RemoveElement(resource,true);
    RDFUnassertAll(gGeneralDatasource,resource,false);
  }
  var sidebar=document.getElementById("sidebar");
  rebuildPage(gGeneralBuilder,sidebar,null);
}


function iterateOverViews(displayType,callback,arg)
{
  var layout = RDFGetResource(gGeneralDatasource,displayType,GENERAL_NS + "displays");
  if (layout){
    var container = RDFGetContainer(gGeneralDatasource,layout);
    if (container){
      var elem = null;
      var iter = container.GetElements();
      while (iter.hasMoreElements()){
        elem = iter.getNext();
        if (elem){
          elem = elem.QueryInterface(Components.interfaces.nsIRDFResource);
          var variant = RDFGetLiteral(gGeneralDatasource,elem,GENERAL_NS + "variant");
          if (variant){
            var retval = callback(elem,variant,arg);
            if (retval) return retval;
          }
        }
      }
    }
  }
  return null;
}

function populateViewList_addMenuItem(elem,variant,list)
{
  if (variant != "top") return null;

  var label = RDFGetLiteral(gGeneralDatasource,elem,GENERAL_NS + "label");
  if (!label) label=variant;

  var menuitem = document.createElement("menuitem");
  menuitem.setAttribute("label",label);
  menuitem.value=elem;
  list.insertBefore(menuitem,list.firstChild);

  return null;
}

function populateViewList()
{
  var list = document.getElementById("menupopup_view");
  while (list.firstChild.tagName != "menuseparator") list.removeChild(list.firstChild);

  var state = getState();
  var resource = getState().resource;
  displayType = RDFGetResource(gGeneralDatasource,resource,RDF_NS + "type");
  while (displayType){
    iterateOverViews(displayType,populateViewList_addMenuItem,list);
    displayType = RDFGetResource(gGeneralDatasource,displayType,RDFS_NS + "subClassOf");
  }
}

// XXX these functions are not currently used

function addSubHistoryItem(id,resource,label)
{
  var RDFContainerUtilsService = "@mozilla.org/rdf/container-utils;1";
  RDFContainerUtilsService = Components.classes[RDFContainerUtilsService].
    getService(Components.interfaces.nsIRDFContainerUtils);

  var childResource=RDF.GetAnonymousResource();

  var container=RDFGetContainer(gGeneralDatasource,resource);
  if (!container){
    container=RDFContainerUtilsService.MakeSeq(gGeneralDatasource,resource);
  }

  container.AppendElement(childResource);
  RDFAssert(gGeneralDatasource,childResource,GENERAL_NS+"id",id);
  RDFAssert(gGeneralDatasource,childResource,GENERAL_NS+"node",resource.Value);

  addToHistory(resource,getState().resource,label);
}

function selectDefaultItem(cdoc)
{
  var container=RDFGetContainer(gGeneralDatasource,getState().historyItem);
  if (container){
    var children=container.GetElements();
    if (children.hasMoreElements()){
      var res=children.getNext();
      var id=RDFGetLiteral(gGeneralDatasource,res,GENERAL_NS+"id");
      var node=RDFGetLiteral(gGeneralDatasource,res,GENERAL_NS+"node");
      var list=cdoc.getElementById(id);
      if (list){
        var rows=list.getRowCount();
        for (var t=0; t<rows; t++){
          var row=list.getItemAtIndex(t);
          if (row.bindID && row.bindID.Value == node){
            list.selectedItem=row;
            list.focus();
            break;
          }
        }
      }
    }
  }
}
